@@ -1,14 +1,16 @@
+# mypy: ignore-errors
 import importlib.util
 import io
 import sys
 from contextlib import redirect_stdout
 from functools import lru_cache
 from pathlib import Path
-from typing import Dict, List, Iterable, Any, Union
+from typing import Dict, List, Iterable, Union
 
 import numpy as np
 import numpy.typing as npt
 import tensorflow as tf  # type: ignore
+import torch  # type: ignore
 
 from code_loader.contract.datasetclasses import DatasetSample, DatasetBaseHandler, InputHandler, \
     GroundTruthHandler, PreprocessResponse, VisualizerHandler, VisualizerCallableReturnType, CustomLossHandler, \
@@ -30,6 +32,9 @@ class LeapLoader:
     @lru_cache()
     def exec_script(self) -> None:
         try:
+            # disable GPU on torch module
+            torch.cuda.is_available = lambda: False
+
             self.evaluate_module()
         except Exception as e:
             raise DatasetScriptException(getattr(e, 'message', repr(e))) from e
@@ -98,13 +103,14 @@ class LeapLoader:
         }
 
     def get_sample(self, state: DataStateEnum, idx: int) -> DatasetSample:
-        self.exec_script()
-        sample = DatasetSample(inputs=self._get_inputs(state, idx),
-                               gt=None if state == DataStateEnum.unlabeled else self._get_gt(state, idx),
-                               metadata=self._get_metadata(state, idx),
-                               index=idx,
-                               state=state)
-        return sample
+        with tf.device('/cpu:0'):
+            self.exec_script()
+            sample = DatasetSample(inputs=self._get_inputs(state, idx),
+                                   gt=None if state == DataStateEnum.unlabeled else self._get_gt(state, idx),
+                                   metadata=self._get_metadata(state, idx),
+                                   index=idx,
+                                   state=state)
+            return sample
 
     def check_dataset(self) -> DatasetIntegParseResult:
         test_payloads: List[DatasetTestResultPayload] = []
